import { createProjectPlugin, ProjectKind } from '@quilted/sewing-kit';

/**
 * Adds basic Babel hooks that other plugins can attach configuration to.
 */
function babelHooks() {
  return createProjectPlugin({
    name: 'SewingKit.Babel',

    build({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    },

    develop({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    },

    test({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    }

  });
}
function babelRollup() {
  return createProjectPlugin({
    name: 'SewingKit.Babel.Rollup',

    build({
      project,
      configure
    }) {
      configure(({
        extensions,
        babelPresets,
        babelPlugins,
        babelTargets,
        babelExtensions,
        rollupPlugins
      }) => {
        rollupPlugins?.(async plugins => {
          const shouldUseRuntime = project.kind === ProjectKind.Package ? 'runtime' : 'bundled';
          const [{
            babel
          }, targets, baseExtensions, babelPresetsOption, babelPluginsOption] = await Promise.all([import('@rollup/plugin-babel'), babelTargets.run([]), extensions.run(['.mjs', '.cjs', '.js']), babelPresets.run([]), babelPlugins.run(shouldUseRuntime ? ['@babel/plugin-transform-runtime'] : [])]);
          const finalExtensions = await babelExtensions.run(baseExtensions);
          return [...plugins, babel({
            envName: 'production',
            extensions: finalExtensions,
            exclude: 'node_modules/**',
            babelHelpers: shouldUseRuntime ? 'runtime' : 'bundled',
            configFile: false,
            babelrc: false,
            skipPreflightCheck: true,
            // @ts-expect-error Babel types have not been updated yet
            targets,
            presets: babelPresetsOption,
            plugins: babelPluginsOption
          })];
        });
      });
    },

    develop({
      project,
      configure
    }) {
      configure(({
        extensions,
        babelPresets,
        babelPlugins,
        babelTargets,
        babelExtensions,
        rollupPlugins
      }) => {
        rollupPlugins?.(async plugins => {
          const shouldUseRuntime = project.kind === ProjectKind.Package ? 'runtime' : 'bundled';
          const [{
            babel
          }, targets, baseExtensions, babelPresetsOption, babelPluginsOption] = await Promise.all([import('@rollup/plugin-babel'), babelTargets.run([]), extensions.run(['.mjs', '.cjs', '.js']), babelPresets.run([]), babelPlugins.run(shouldUseRuntime ? ['@babel/plugin-transform-runtime'] : [])]);
          const finalExtensions = await babelExtensions.run(baseExtensions);
          return [...plugins, babel({
            envName: 'production',
            extensions: finalExtensions,
            exclude: 'node_modules/**',
            babelHelpers: shouldUseRuntime ? 'runtime' : 'bundled',
            configFile: false,
            babelrc: false,
            skipPreflightCheck: true,
            // @ts-expect-error Babel types have not been updated yet
            targets,
            presets: babelPresetsOption,
            plugins: babelPluginsOption
          })];
        });
      });
    }

  });
}

export { babelHooks, babelRollup };
